@@ -18,7 +18,7 @@
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * or visit http://www.gnu.org/copyleft/gpl.html
 *
-* 20010215 raf <raf@raf.org>
+* 20011109 raf <raf@raf.org>
 */
 
 /*
@@ -60,18 +60,17 @@ I<libslack(mem)> - memory module
     #define mem_destroy4d(space)
     #define mem_destroy_space(space)
     Pool *pool_create(size_t size);
-    Pool *pool_create_locked(Locker *locker, size_t size);
+    Pool *pool_create_with_locker(Locker *locker, size_t size);
     void pool_release(Pool *pool);
     void *pool_destroy(Pool **pool);
     Pool *pool_create_secure(size_t size);
-    Pool *pool_create_secure_locked(Locker *locker, size_t size);
+    Pool *pool_create_secure_with_locker(Locker *locker, size_t size);
     void pool_release_secure(Pool *pool);
-    #define pool_destroy_secure(pool)
-    void *pool_destroy_secure_fn(Pool **pool);
+    void *pool_destroy_secure(Pool **pool);
     void pool_clear_secure(Pool *pool);
     #define pool_new(pool, type)
     #define pool_newsz(pool, size, type)
-    void *pool_alloc(Pool *pool, size_t bytes);
+    void *pool_alloc(Pool *pool, size_t size);
     void pool_clear(Pool *pool);
 
 
@@ -79,7 +78,7 @@ I<libslack(mem)> - memory module
 
 This module is mostly just an interface to I<malloc(3)>, I<realloc(3)> and
 I<free(3)> that tries to ensure that pointers that don't point to anything
-get set to C<NULL>. It also provides dynamically allocated multi-dimensional
+get set to C<null>. It also provides dynamically allocated multi-dimensional
 arrays, memory pools and secure memory for the more adventurous.
 
 =over 4
@@ -88,9 +87,10 @@ arrays, memory pools and secure memory for the more adventurous.
 
 */
 
+#include "config.h"
 #include "std.h"
 
-#ifdef _POSIX_MEMLOCK_RANGE
+#ifdef HAVE_MLOCK
 #include <sys/mman.h>
 #endif
 
@@ -105,11 +105,13 @@ struct Pool
 	Locker *locker; /* locking strategy for the pool */
 };
 
+#ifndef TEST
+
 /*
 
 =item C< #define null NULL>
 
-A less angular version of C<NULL>.
+Easier to type. Easier to read.
 
 =item C< #define nul '\0'>
 
@@ -118,71 +120,77 @@ A name for the C<nul> character.
 =item C< #define mem_new(type)>
 
 Allocates enough memory (with I<malloc(3)>) to store an object of type
-C<type>. On success, returns the address of the allocated memory. On error,
-returns C<NULL>.
+C<type>. It is the caller's responsibility to deallocate the allocated
+memory with I<mem_release()>, I<mem_destroy()> or I<free(3)>. On success,
+returns the address of the allocated memory. On error, returns C<null>.
 
 =item C< #define mem_create(size, type)>
 
 Allocates enough memory (with I<malloc(3)>) to store C<size> objects of type
-C<type>. On success, returns the address of the allocated memory. On error,
-returns C<NULL>.
+C<type>. It is the caller's responsibility to deallocate the allocated
+memory with I<mem_release()>, I<mem_destroy()> or I<free(3)>. On success,
+returns the address of the allocated memory. On error, returns C<null>.
 
 =item C< #define mem_resize(mem, num)>
 
-Alters the amount of memory pointed to by C<*mem>. If C<*mem> is C<NULL>,
-I<malloc(3)> is used to allocate new memory. If size is zero, I<free(3)> is
-called to deallocate the memory and C<*mem> is set to C<NULL>. Otherwise,
-I<realloc(3)> is called. If I<realloc(3)> needs to allocate new memory to
-satisfy a request, C<*mem> is set to the new address. On success, returns
-I<*mem> (though it may be C<NULL> if C<size> is zero). On error, C<NULL> is
-returned and I<*mem> is not altered.
+Alters the amount of memory pointed to by C<*mem>. If C<*mem> is C<null>,
+new memory is allocated and assigned to C<*mem>. If size is zero, C<*mem> is
+deallocated and C<null> is assigned to C<*mem>. Otherwise, C<*mem> is
+reallocated and assigned back to C<*mem>. On success, returns C<*mem> (which
+might be C<null>). On error, returns C<null> with C<errno> set appropriately
+and C<*mem> is not altered.
 
 =item C<void *mem_resize_fn(void **mem, size_t size)>
 
-A single interface for altering the size of allocated memory. C<mem> points
-to the pointer to be affected. C<size> is the size in bytes of the memory
-that this pointer is to point to. If the pointer is C<NULL>, I<malloc(3)> is
-used to obtain memory. If C<size> is zero, I<free(3)> is used to release the
-memory. In all other cases, I<realloc(3)> is used to alter the size of the
-memory. In all cases, the pointer pointed to by C<mem> is assigned to the
-memory's location (or C<NULL> when C<size> is zero). This function is
-exposed as an implementation side effect. Don't call it directly. Call
-I<mem_resize()> instead. On error (i.e. I<malloc(3)> or I<realloc(3)> fail
-or C<mem> is C<NULL>), returns C<NULL> without setting C<*mem> to anything.
+An interface to I<realloc(3)> that also assigns to a pointer variable unless
+an error occurred. C<mem> points to the pointer to be affected. C<size> is
+the requested size in bytes. If C<size> is zero, C<*mem> is deallocated and
+set to C<null>. This function is exposed as an implementation side effect.
+Don't call it directly. Call I<mem_resize()> instead. On error, returns
+C<null> with C<errno> set appropriately.
 
 =cut
 
 */
 
-void *mem_resize_fn(void **mem, size_t size)
+#ifdef HAVE_ISOC_REALLOC
+
+#define isoc_realloc realloc
+
+#else
+
+static void *isoc_realloc(void *ptr, size_t size)
 {
 	void *p;
 
-	if (!mem)
-		return NULL;
-
 	if (size)
 	{
-		if (*mem)
-		{
-			p = realloc(*mem, size);
-			if (p)
-				return *mem = p;
-		}
-		else
-		{
-			p = malloc(size);
-			if (p)
-				return *mem = p;
-		}
+		if (!(p = (ptr) ? realloc(ptr, size) : malloc(size)))
+			errno = ENOMEM; /* Not required by ISO C but handy */
 	}
 	else
 	{
-		free(*mem);
-		return *mem = NULL;
+		free(ptr);
+		p = NULL;
 	}
 
-	return NULL;
+	return p;
+}
+
+#endif
+
+void *mem_resize_fn(void **mem, size_t size)
+{
+	void *ptr;
+
+	if (!mem)
+		return set_errnull(EINVAL);
+
+	ptr = isoc_realloc(*mem, size);
+	if (size && !ptr)
+		return NULL;
+
+	return *mem = ptr;
 }
 
 /*
@@ -191,12 +199,12 @@ void *mem_resize_fn(void **mem, size_t size)
 
 Releases (deallocates) C<mem>. Same as I<free(3)>. Only to be used in
 destructor functions. In other cases, use I<mem_destroy()> which also sets
-C<mem> to C<NULL>.
+C<mem> to C<null>.
 
 =item C<void *mem_destroy(void **mem)>
 
-Calls I<free(3)> on the pointer pointed to by C<*mem>. Then assigns C<NULL>
-to this pointer. Returns C<NULL>.
+Calls I<free(3)> on the pointer, C<*mem>. Then assigns C<null> to this
+pointer. Returns C<null>.
 
 =cut
 
@@ -219,24 +227,24 @@ void *(mem_destroy)(void **mem)
 
 Allocates C<size> bytes of memory (with I<malloc(3)>) and then locks it into
 RAM with I<mlock(2)> so that it can't be paged to disk where some nefarious
-local user with root access might read its contents. The memory returned must
-only be deallocated using I<mem_release_secure()> or I<mem_destroy_secure()>
-which will clear the memory and unlock it before deallocating it. On success,
-returns the address of the secure allocated memory. On error, returns C<NULL>
-with C<errno> set appropriately.
+local user with root access might read its contents. It is the caller's
+responsibility to deallocate the secure memory with I<mem_release_secure()>
+or I<mem_destroy_secure()> which will clear the memory and unlock it before
+deallocating it. On success, returns the address of the secure allocated
+memory. On error, returns C<null> with C<errno> set appropriately.
 
 Note that entire pages are locked by I<mlock(2)> so don't create many, small
 pieces of secure memory or many entire pages will be locked. Use a secure
 memory pool instead. Also note that secure memory requires root privileges.
 
 On some systems (e.g. Solaris), memory locks must start on page boundaries.
-So we need to C<malloc()> enough memory to extend from whatever address
-C<malloc()> may return to the next page boundary (worst case: C<pagesize -
+So we need to I<malloc()> enough memory to extend from whatever address
+I<malloc()> may return to the next page boundary (worst case: C<pagesize -
 sizeof(int)>) and then the actual number of bytes requested. We need an
-additional 8 bytes to store the address returned by C<malloc()> (so we can
-C<free()> it later) and the size passed to C<mlock()> so we can pass it to
-C<munlock()> later. Unfortunately, we need to store the address and size
-after the page boundary and not before it because C<malloc()> may return a
+additional 8 bytes to store the address returned by I<malloc()> (so we can
+I<free()> it later) and the size passed to I<mlock()> so we can pass it to
+I<munlock()> later. Unfortunately, we need to store the address and size
+after the page boundary and not before it because I<malloc()> may return a
 page boundary or an address less than 8 bytes to the left of a page
 boundary.
 
@@ -253,7 +261,7 @@ It will look like:
    +- malloc()             +- address returned
 
 If your system doesn't require page boundaries (e.g. Linux), the address
-returned by C<malloc()> is locked and returned and only the size is stored.
+returned by I<malloc()> is locked and returned and only the size is stored.
 
 =cut
 
@@ -262,18 +270,14 @@ returned by C<malloc()> is locked and returned and only the size is stored.
 
 void *mem_create_secure(size_t size)
 {
-#ifdef _POSIX_MEMLOCK_RANGE
+#ifdef HAVE_MLOCK
 
 	char *addr, *lock;
-#ifdef MLOCK_NEEDS_PAGE_BOUNDARY
+#ifdef MLOCK_REQUIRES_PAGE_BOUNDARY
 	long pagesize;
 
-	pagesize = sysconf(_SC_PAGESIZE);
-	if (pagesize == -1)
-	{
-		set_errno(EINVAL);
-		return NULL;
-	}
+	if ((pagesize = sysconf(_SC_PAGESIZE)) == -1)
+		return set_errnull(EINVAL);
 
 	size += sizeof(void *) + sizeof(size_t);
 	addr = malloc(pagesize - sizeof(int) + size);
@@ -285,7 +289,7 @@ void *mem_create_secure(size_t size)
 	if (!addr)
 		return NULL;
 
-#ifdef MLOCK_NEEDS_PAGE_BOUNDARY
+#ifdef MLOCK_REQUIRES_PAGE_BOUNDARY
 	if ((long)addr & (pagesize - 1)) /* addr not on page boundary */
 		lock = (void *)(((long)addr & ~(pagesize - 1)) + pagesize);
 	else
@@ -300,7 +304,7 @@ void *mem_create_secure(size_t size)
 		return NULL;
 	}
 
-#ifdef MLOCK_NEEDS_PAGE_BOUNDARY
+#ifdef MLOCK_REQUIRES_PAGE_BOUNDARY
 	*(void **)lock = addr;
 	lock += sizeof(void *);
 #endif
@@ -318,10 +322,10 @@ void *mem_create_secure(size_t size)
 
 =item C<void mem_release_secure(void *mem)>
 
-Sets the contents of C<mem> to C<nul> bytes, then unlocks and releases
-(deallocates) C<mem>. Only to be used on memory returned by
+Sets the memory pointed to by C<mem> to C<nul> bytes, then unlocks and
+releases (deallocates) C<mem>. Only to be used on memory returned by
 I<mem_create_secure()>. Only to be used in destructor functions. In other
-cases, use I<mem_destroy()> which also sets C<mem> to C<NULL>.
+cases, use I<mem_destroy()> which also sets C<mem> to C<null>.
 
 =cut
 
@@ -329,7 +333,7 @@ cases, use I<mem_destroy()> which also sets C<mem> to C<NULL>.
 
 void mem_release_secure(void *mem)
 {
-#ifdef _POSIX_MEMLOCK_RANGE
+#ifdef HAVE_MLOCK
 	char *addr, *lock;
 	size_t size;
 
@@ -339,14 +343,17 @@ void mem_release_secure(void *mem)
 	lock = mem;
 	lock -= sizeof(size_t);
 	size = *(size_t *)lock;
-#ifdef MLOCK_NEEDS_PAGE_BOUNDARY
+#ifdef MLOCK_REQUIRES_PAGE_BOUNDARY
 	lock -= sizeof(void *);
 	addr = *(void **)lock;
 #else
 	addr = lock;
 #endif
 
-	memset(lock, nul, size);
+	memset(lock, 0xff, size);
+	memset(lock, 0xaa, size);
+	memset(lock, 0x55, size);
+	memset(lock, 0x00, size);
 	munlock(lock, size);
 	free(addr);
 #endif
@@ -356,9 +363,9 @@ void mem_release_secure(void *mem)
 
 =item C<void *mem_destroy_secure(void **mem)>
 
-Sets the contents of C<*mem> to C<nul> bytes, then unlocks and destroys
-(deallocates and sets to C<NULL>) it. Only to be used on memory returned by
-I<mem_create_secure()>. Returns C<NULL>.
+Sets the memory pointed to by C<*mem> to C<nul> bytes, then unlocks and
+destroys (deallocates and sets to C<null>) C<*mem>. Only to be used on
+memory returned by I<mem_create_secure()>. Returns C<null>.
 
 =cut
 
@@ -379,8 +386,10 @@ void *(mem_destroy_secure)(void **mem)
 
 =item C<char *mem_strdup(const char *str)>
 
-Returns a dynamically allocated copy of C<str>. On error, returns C<NULL>.
-The caller must deallocate the memory returned.
+Returns a dynamically allocated copy of C<str>. It is the caller's
+responsibility to deallocate the new string with I<mem_release()>,
+I<mem_destroy()> or I<free(3)>. On error, returns C<null> with C<errno> set
+appropriately.
 
 =cut
 
@@ -392,12 +401,10 @@ char *mem_strdup(const char *str)
 	char *copy;
 
 	if (!str)
-		return NULL;
+		return set_errnull(EINVAL);
 
 	size = strlen(str) + 1;
-	copy = mem_create(size, char);
-
-	if (!copy)
+	if (!(copy = mem_create(size, char)))
 		return NULL;
 
 	return memcpy(copy, str, size);
@@ -407,21 +414,21 @@ char *mem_strdup(const char *str)
 
 =item C< #define mem_create2d(i, j, type)>
 
-Shorthand for allocating a 2-dimensional array. See I<mem_create_space()>.
+Alias for allocating a 2-dimensional array. See I<mem_create_space()>.
 
 =item C< #define mem_create3d(i, j, k, type)>
 
-Shorthand for allocating a 3-dimensional array. See I<mem_create_space()>.
+Alias for allocating a 3-dimensional array. See I<mem_create_space()>.
 
 =item C< #define mem_create4d(i, j, k, l, type)>
 
-Shorthand for allocating a 4-dimensional array. See I<mem_create_space()>.
+Alias for allocating a 4-dimensional array. See I<mem_create_space()>.
 
 =item C<void *mem_create_space(size_t size, ...)>
 
 Allocates a multi-dimensional array of elements of size C<size> and sets the
-memory to zero. The remaining arguments specify the sizes of each dimension.
-The last argument must be zero. There is an arbitrary limit of 32
+memory to C<nul> bytes. The remaining arguments specify the sizes of each
+dimension. The last argument must be zero. There is an arbitrary limit of 32
 dimensions. The memory returned is set to zero. The memory returned needs to
 be cast or assigned into the appropriate pointer type. You can then set and
 access elements exactly like a real multi-dimensional C array. Finally, it
@@ -475,8 +482,7 @@ void *mem_create_space(size_t size, ...)
 		dim[d] = arg;
 	va_end(args);
 
-	length = 0;
-	for (i = 0; i < d; ++i)
+	for (length = i = 0; i < d; ++i)
 	{
 		starts[i] = length;
 		lengths[i] = sizes[i] = (i == d - 1) ? size : sizeof(void *);
@@ -530,8 +536,7 @@ size_t mem_space_start(size_t size, ...)
 		dim[d] = arg;
 	va_end(args);
 
-	length = 0;
-	for (i = 0; i < d; ++i)
+	for (length = i = 0; i < d; ++i)
 	{
 		lengths[i] = (i == d - 1) ? size : sizeof(void *);
 		for (j = 0; j <= i; ++j)
@@ -564,26 +569,26 @@ See I<mem_release_space()>.
 Releases (deallocates) a multi-dimensional array, C<space>, allocated with
 I<mem_create_space>. Same as I<free(3)>. Only to be used in destructor
 functions. In other cases, use I<mem_destroy_space()> which also sets
-C<space> to C<NULL>.
+C<space> to C<null>.
 
 =item C< #define mem_destroy2d(space)>
 
-Alias for destroying (deallocating and setting to C<NULL>) a
+Alias for destroying (deallocating and setting to C<null>) a
 2-dimensional array. See I<mem_destroy_space()>.
 
 =item C< #define mem_destroy3d(space)>
 
-Alias for destroying (deallocating and setting to C<NULL>) a
+Alias for destroying (deallocating and setting to C<null>) a
 3-dimensional array. See I<mem_destroy_space()>.
 
 =item C< #define mem_destroy4d(space)>
 
-Alias for destroying (deallocating and setting to C<NULL>) a
+Alias for destroying (deallocating and setting to C<null>) a
 4-dimensional array. See I<mem_destroy_space()>.
 
 =item C< #define mem_destroy_space(mem)>
 
-Destroys (deallocates and sets to C<NULL>) the multi-dimensional array
+Destroys (deallocates and sets to C<null>) the multi-dimensional array
 pointed to by C<space>.
 
 =cut
@@ -594,16 +599,16 @@ pointed to by C<space>.
 
 =item C<Pool *pool_create(size_t size)>
 
-Creates a memory pool of size C<size> from which many smaller chunks of
-memory may be subsequently allocated (with I<pool_alloc()>) without
-resorting to the use of C<malloc(3)>. Useful when you have many small
-objects to allocate but I<malloc(3)> is slowing your program down too much.
-On success, returns the pool. On error, returns C<NULL>.
+Creates a memory pool of size C<size> from which smaller chunks of memory
+may be subsequently allocated (with I<pool_alloc()>) without resorting to
+the use of I<malloc(3)>. Useful when you have many small objects to allocate
+but I<malloc(3)> is slowing your program down too much. It is the caller's
+responsibility to deallocate the new pool with I<pool_release()> or
+I<pool_destroy()>. On success, returns the pool. On error, returns C<null>.
 
 The size of a pool can't be changed after it is created and the individual
 chunks of memory allocated from within a pool can't be separately
-deallocated. The entire pool can be emptied with I<pool_clear()> and the
-pool can be deallocated with I<pool_release()> or I<pool_destroy()>.
+deallocated. The entire pool can be emptied with I<pool_clear()>.
 
 =cut
 
@@ -611,22 +616,21 @@ pool can be deallocated with I<pool_release()> or I<pool_destroy()>.
 
 Pool *pool_create(size_t size)
 {
-	return pool_create_locked(NULL, size);
+	return pool_create_with_locker(NULL, size);
 }
 
 /*
 
-=item C<Pool *pool_create_locked(Locker *locker, size_t size)>
+=item C<Pool *pool_create_with_locker(Locker *locker, size_t size)>
 
-Just like I<pool_alloc()> except that multiple threads accessing this pool
-will be synchronised by C<locker>. On success, returns the pool. On error,
-returns C<NULL>.
+Equivalent to I<pool_create()> except that multiple threads accessing the
+new pool will be synchronised by C<locker>.
 
 =cut
 
 */
 
-Pool *pool_create_locked(Locker *locker, size_t size)
+Pool *pool_create_with_locker(Locker *locker, size_t size)
 {
 	Pool *pool = mem_create(1, Pool);
 
@@ -651,35 +655,24 @@ Pool *pool_create_locked(Locker *locker, size_t size)
 C<int pool_lock(Pool *pool)>
 
 Claims a write lock on C<pool>. On success, returns C<0>. On error, returns
-C<-1>.
-
-*/
-
-static int pool_lock(Pool *pool)
-{
-	return locker_wrlock(pool->locker);
-}
-
-/*
+an error code.
 
 C<int pool_unlock(Pool *pool)>
 
 Unlocks a write lock on C<pool>. On success, returns C<0>. On error, returns
-C<-1>.
+an error code.
 
 */
 
-static int pool_unlock(Pool *pool)
-{
-	return locker_unlock(pool->locker);
-}
+#define pool_lock(pool) locker_wrlock((pool)->locker)
+#define pool_unlock(pool) locker_unlock((pool)->locker)
 
 /*
 
 =item C<void pool_release(Pool *pool)>
 
 Releases (deallocates) C<pool>. Only to be used in destructor functions. In
-other cases, use I<pool_destroy()> which also sets C<pool> to C<NULL>.
+other cases, use I<pool_destroy()> which also sets C<pool> to C<null>.
 
 =cut
 
@@ -688,26 +681,32 @@ other cases, use I<pool_destroy()> which also sets C<pool> to C<NULL>.
 void pool_release(Pool *pool)
 {
 	Locker *locker;
+	int err;
 
 	if (!pool)
 		return;
 
-	if (pool_lock(pool) == -1)
+	if ((err = pool_lock(pool)))
+	{
+		set_errno(err);
 		return;
+	}
 
 	locker = pool->locker;
 	mem_release(pool->pool);
 	mem_release(pool);
-	locker_unlock(locker);
+
+	if ((err = locker_unlock(locker)))
+		set_errno(err);
 }
 
 /*
 
 =item C<void *pool_destroy(Pool **pool)>
 
-Destroys (deallocates and sets to C<NULL>) C<*pool>. Returns C<NULL>.
+Destroys (deallocates and sets to C<null>) C<*pool>. Returns C<null>.
 B<Note:> pools shared by multiple threads must not be destroyed until after
-the threads have finished with it.
+all threads have finished with it.
 
 =cut
 
@@ -729,14 +728,14 @@ void *pool_destroy(Pool **pool)
 =item C<Pool *pool_create_secure(size_t size)>
 
 Creates a memory pool of size C<size> just like I<pool_create()> except that
-the memory pool itself is locked into RAM with I<mlock(2)> so that it can't
-be paged to disk where some nefarious local user might read its contents.
-The pool returned must only be deallocated using I<pool_release_secure()> or
-I<pool_destroy_secure()> which will clear the memory pool and unlock it
-before deallocating it. In all other ways, the pool returned is exactly like
-a pool returned by I<pool_create()>. On success, returns the pool. On error,
-returns C<NULL> with C<errno> set appropriately. Note that secure memory
-requires root privileges.
+the memory pool is locked into RAM with I<mlock(2)> so that it can't be
+paged to disk where some nefarious local user might read its contents. It is
+the caller's responsibility to deallocate the new pool with
+I<pool_release_secure()> or I<pool_destroy_secure()> which will clear the
+memory pool and unlock it before deallocating it. In all other ways, the
+pool returned is exactly like a pool returned by I<pool_create()>. On
+success, returns the pool. On error, returns C<null> with C<errno> set
+appropriately. Note that secure memory requires root privileges.
 
 =cut
 
@@ -744,23 +743,23 @@ requires root privileges.
 
 Pool *pool_create_secure(size_t size)
 {
-	return pool_create_secure_locked(NULL, size);
+	return pool_create_secure_with_locker(NULL, size);
 }
 
 /*
 
-=item C<Pool *pool_create_secure_locked(Locker *locker, size_t size)>
+=item C<Pool *pool_create_secure_with_locker(Locker *locker, size_t size)>
 
-Just like I<pool_create_secure()> except that multiple threads accessing
-this pool will be synchronised by I<locker>.
+Equivalent to I<pool_create_secure()> except that multiple threads accessing
+the new pool will be synchronised by C<locker>.
 
 =cut
 
 */
 
-Pool *pool_create_secure_locked(Locker *locker, size_t size)
+Pool *pool_create_secure_with_locker(Locker *locker, size_t size)
 {
-#ifdef _POSIX_MEMLOCK_RANGE
+#ifdef HAVE_MLOCK
 	Pool *pool = mem_create(1, Pool);
 
 	if (!pool)
@@ -790,7 +789,7 @@ Pool *pool_create_secure_locked(Locker *locker, size_t size)
 Sets the contents of the memory pool to C<nul> bytes, then unlocks and
 releases (deallocates) C<pool>. Only to be used on pools returned by
 I<pool_create_secure()>. Only to be used in destructor functions. In other
-cases, use I<pool_destroy_secure()> which also sets C<pool> to C<NULL>.
+cases, use I<pool_destroy_secure()> which also sets C<pool> to C<null>.
 
 =cut
 
@@ -798,19 +797,25 @@ cases, use I<pool_destroy_secure()> which also sets C<pool> to C<NULL>.
 
 void pool_release_secure(Pool *pool)
 {
-#ifdef _POSIX_MEMLOCK_RANGE
+#ifdef HAVE_MLOCK
 	Locker *locker;
+	int err;
 
 	if (!pool)
 		return;
 
-	if (pool_lock(pool) == -1)
+	if ((err = pool_lock(pool)))
+	{
+		set_errno(err);
 		return;
+	}
 
 	locker = pool->locker;
 	mem_release_secure(pool->pool);
 	mem_release(pool);
-	locker_unlock(locker);
+
+	if ((err = locker_unlock(locker)))
+		set_errno(err);
 #endif
 }
 
@@ -819,9 +824,9 @@ void pool_release_secure(Pool *pool)
 =item C<void *pool_destroy_secure(Pool **pool)>
 
 Sets the contents of the memory pool to C<nul> bytes, then unlocks and
-destroyd (deallocates and sets to C<NULL>) C<*pool). Returns C<NULL>.
+destroys (deallocates and sets to C<null>) C<*pool>. Returns C<null>.
 B<Note:> secure pools shared by multiple threads must not be destroyed until
-after the threads have finished with it.
+after all threads have finished with it.
 
 =cut
 
@@ -854,15 +859,25 @@ static void pool_clear_unlocked(Pool *pool);
 
 void pool_clear_secure(Pool *pool)
 {
+	int err;
+
 	if (!pool)
 		return;
 
-	if (pool_lock(pool) == -1)
+	if ((err = pool_lock(pool)))
+	{
+		set_errno(err);
 		return;
+	}
 
 	pool_clear_unlocked(pool);
-	memset(pool->pool, nul, pool->size);
-	pool_unlock(pool);
+	memset(pool->pool, 0xff, pool->size);
+	memset(pool->pool, 0xaa, pool->size);
+	memset(pool->pool, 0x55, pool->size);
+	memset(pool->pool, 0x00, pool->size);
+
+	if ((err = pool_unlock(pool)))
+		set_errno(err);
 }
 
 /*
@@ -871,25 +886,25 @@ void pool_clear_secure(Pool *pool)
 
 Allocates enough memory from C<pool> to store an object of type C<type>. On
 success, returns the address of the allocated memory. On error, returns
-C<NULL> with C<errno> set appropriately.
+C<null> with C<errno> set appropriately.
 
 =item C< #define pool_newsz(pool, size, type)>
 
 Allocates enough memory from C<pool> to store C<size> objects of type
 C<type>. On success, returns the address of the allocated memory. On error,
-returns C<NULL> with C<errno> set appropriately.
+returns C<null> with C<errno> set appropriately.
 
 =item C<void *pool_alloc(Pool *pool, size_t size)>
 
 Allocates a chunk of memory of C<size> bytes from C<pool>. Does not use
-C<malloc(3)>. The pointer returned must not be passed to I<free(3)> or
-C<realloc(3)>. Only the entire pool can be deallocated with
+I<malloc(3)>. The pointer returned must not be passed to I<free(3)> or
+I<realloc(3)>. Only the entire pool can be deallocated with
 I<pool_release()> or I<pool_destroy()>. All of the chunks can be deallocated
 in one go with I<pool_clear()> without deallocating the pool itself.
 
 On success, returns the pointer to the allocated pool memory. On error,
-returns C<NULL> with C<errno> set appropriately (i.e. C<EINVAL> if C<pool>
-is C<NULL>, I<ENOSPC> if C<pool> does not have enough unused memory to
+returns C<null> with C<errno> set appropriately (i.e. C<EINVAL> if C<pool>
+is C<null>, C<ENOSPC> if C<pool> does not have enough unused memory to
 allocate C<size> bytes).
 
 It is the caller's responsibility to ensure the correct alignment if
@@ -904,28 +919,25 @@ specific alignment.
 void *pool_alloc(Pool *pool, size_t size)
 {
 	void *addr;
+	int err;
 
 	if (!pool)
-	{
-		set_errno(EINVAL);
-		return NULL;
-	}
+		return set_errnull(EINVAL);
 
-	if (pool_lock(pool) == -1)
-		return NULL;
+	if ((err = pool_lock(pool)))
+		return set_errnull(err);
 
 	if (pool->used + size > pool->size)
 	{
-		set_errno(ENOSPC);
 		pool_unlock(pool);
-		return NULL;
+		return set_errnull(ENOSPC);
 	}
 
 	addr = pool->pool + pool->used;
 	pool->used += size;
 
-	if (pool_unlock(pool) == -1)
-		return NULL;
+	if ((err = pool_unlock(pool)))
+		return set_errnull(err);
 
 	return addr;
 }
@@ -941,39 +953,66 @@ that it can be reused. Does not use I<free(3)>.
 
 */
 
-static void pool_clear_locked(Pool *pool, int lock_pool)
+static void pool_clear_with_locker(Pool *pool, int lock_pool)
 {
+	int err;
+
 	if (!pool)
 		return;
 
-	if (lock_pool && pool_lock(pool) == -1)
+	if (lock_pool && (err = pool_lock(pool)))
+	{
+		set_errno(err);
 		return;
+	}
 
 	pool->used = 0;
 
-	if (lock_pool)
-		pool_unlock(pool);
+	if (lock_pool && (err = pool_unlock(pool)))
+		set_errno(err);
 }
 
 static void pool_clear_unlocked(Pool *pool)
 {
-	pool_clear_locked(pool, 0);
+	pool_clear_with_locker(pool, 0);
 }
 
 void pool_clear(Pool *pool)
 {
-	pool_clear_locked(pool, 1);
+	pool_clear_with_locker(pool, 1);
 }
 
 /*
 
 =back
 
+=head1 ERRORS
+
+On error, C<errno> is set by underlying functions or as follows:
+
+=over 4
+
+=item EINVAL
+
+When arguments are invalid.
+
+=item ENOSPC
+
+When there is insufficient available space in a pool for I<pool_alloc()>
+to satisfy a request.
+
+=item ENOSYS
+
+Returned by I<mem_create_secure()> and I<pool_create_secure()> when
+I<mlock()> is not supported (e.g. Mac OS X).
+
+=back
+
 =head1 MT-Level
 
 MT-Safe (mem)
 
-MT-Disciplined (pool) man I<thread(3)> for details.
+MT-Disciplined (pool) man I<locker(3)> for details.
 
 =head1 EXAMPLES
 
@@ -1002,13 +1041,14 @@ A pool of a million integers:
         Pool *pool;
         int i, *p;
 		
-        pool = pool_create(1024 * 1024 * sizeof(int));
-        if (!pool)
+        if (!(pool = pool_create(1024 * 1024 * sizeof(int))))
             return;
 
         for (i = 0; i < 1024 * 1024; ++i)
         {
-            p = pool_new(pool, int);
+            if (!(p = pool_new(pool, int)))
+                break;
+
             *p = i;
         }
 
@@ -1018,8 +1058,10 @@ A pool of a million integers:
 Secure memory:
 
     char *secure_passwd = mem_create_secure(32);
+
     if (!secure_passwd)
-        exit(1);
+        exit(EXIT_FAILURE);
+
     get_passwd(secure_passwd, 32);
     use_passwd(secure_passwd);
     mem_destroy_secure(&secure_passwd);
@@ -1028,9 +1070,10 @@ Secure memory pool:
 
     Pool *secure_pool;
     char *secure_passwd;
-    secure_pool = pool_create_secure(1024 * 1024);
-    if (!secure_pool)
-        exit(1);
+
+    if (!(secure_pool = pool_create_secure(1024 * 1024)))
+        exit(EXIT_FAILURE);
+
     secure_passwd = pool_alloc(secure_pool, 32);
     get_passwd(secure_passwd, 32);
     use_passwd(secure_passwd);
@@ -1044,16 +1087,19 @@ L<realloc(3)|realloc(3)>,
 L<calloc(3)|calloc(3)>,
 L<free(3)|free(3)>,
 L<mlock(2)|mlock(2)>,
-L<thread(3)|thread(3)>
+L<munlock(2)|munlock(2)>,
+L<locker(3)|locker(3)>
 
 =head1 AUTHOR
 
-20010215 raf <raf@raf.org>
+20011109 raf <raf@raf.org>
 
 =cut
 
 */
 
+#endif
+
 #ifdef TEST
 
 #include <time.h>
@@ -1078,50 +1124,38 @@ int main(int ac, char **av)
 	int errors = 0;
 	int no_secure_mem;
 
+	if (ac == 2 && !strcmp(av[1], "help"))
+	{
+		printf("usage: %s [pool]\n", *av);
+		return EXIT_SUCCESS;
+	}
+
 	printf("Testing: mem\n");
 
 	/* Test create, resize and destroy */
 
-	mem1 = mem_create(100, int);
-	if (!mem1)
+	if (!(mem1 = mem_create(100, int)))
 		++errors, printf("Test1: mem_create(100, int) failed\n");
-
-	mem_resize(&mem1, 50);
-	if (!mem1)
+	if (!(mem_resize(&mem1, 50)))
 		++errors, printf("Test2: mem_resize(100 -> 50) failed\n");
-
-	mem_resize(&mem1, 0);
-	if (mem1)
+	if (mem_resize(&mem1, 0))
 		++errors, printf("Test3: mem_resize(50 -> 0) failed\n");
-
-	mem_resize(&mem1, 50);
-	if (!mem1)
+	if (!(mem_resize(&mem1, 50)))
 		++errors, printf("Test4: mem_resize(0 -> 50) failed\n");
-
-	mem_destroy(&mem1);
-	if (mem1)
+	if (mem_destroy(&mem1))
 		++errors, printf("Test5: mem_destroy() failed\n");
-
-	mem2 = mem_create(0, char);
-	if (!mem2)
+	if (!(mem2 = mem_create(0, char)))
 		++errors, printf("Test6: mem_create(0, char) failed\n");
-
-	mem_resize(&mem2, 10);
-	if (!mem2)
+	if (!(mem_resize(&mem2, 10)))
 		++errors, printf("Test7: mem_resize(0 -> 10) failed\n");
-
-	mem_resize(&mem2, 0);
-	if (mem2)
+	if (mem_resize(&mem2, 0))
 		++errors, printf("Test8: mem_resize(10 -> 0) failed\n");
-
-	mem_destroy(&mem2);
-	if (mem2)
+	if (mem_destroy(&mem2))
 		++errors, printf("Test9: mem_destroy() failed\n");
 
 	/* Test strdup */
 
-	str = mem_strdup("test");
-	if (!str)
+	if (!(str = mem_strdup("test")))
 		++errors, printf("Test10: mem_strdup() failed (returned NULL)\n");
 	else
 	{
@@ -1133,8 +1167,7 @@ int main(int ac, char **av)
 
 	/* Test 2D space allocation and deallocation */
 
-	space2 = mem_create_space(sizeof(int), 1, 1, 0);
-	if (!space2)
+	if (!(space2 = mem_create_space(sizeof(int), 1, 1, 0)))
 		++errors, printf("Test12: mem_create_space(1, 1) failed (returned NULL)\n");
 	else
 	{
@@ -1147,8 +1180,7 @@ int main(int ac, char **av)
 			++errors, printf("Test14: mem_destroy_space(1, 1) failed\n");
 	}
 
-	space2 = mem_create_space(sizeof(int), 10, 10, 0);
-	if (!space2)
+	if (!(space2 = mem_create_space(sizeof(int), 10, 10, 0)))
 		++errors, printf("Test15: mem_create_space(10, 10) failed (returned NULL)\n");
 	else
 	{
@@ -1173,8 +1205,7 @@ int main(int ac, char **av)
 
 	/* Test 3D space allocation and deallocation */
 
-	space3 = mem_create_space(sizeof(int), 1, 1, 1, 0);
-	if (!space3)
+	if (!(space3 = mem_create_space(sizeof(int), 1, 1, 1, 0)))
 		++errors, printf("Test18: mem_create_space(1, 1, 1) failed (returned NULL)\n");
 	else
 	{
@@ -1187,8 +1218,7 @@ int main(int ac, char **av)
 			++errors, printf("Test20: mem_destroy_space(1, 1, 1) failed\n");
 	}
 
-	space3 = mem_create_space(sizeof(int), 10, 10, 10, 0);
-	if (!space3)
+	if (!(space3 = mem_create_space(sizeof(int), 10, 10, 10, 0)))
 		++errors, printf("Test21: mem_create_space(10, 10, 10) failed (returned NULL)\n");
 	else
 	{
@@ -1215,8 +1245,7 @@ int main(int ac, char **av)
 
 	/* Test 4D space allocation and deallocation */
 
-	space4 = mem_create_space(sizeof(int), 1, 1, 1, 1, 0);
-	if (!space4)
+	if (!(space4 = mem_create_space(sizeof(int), 1, 1, 1, 1, 0)))
 		++errors, printf("Test24: mem_create_space(1, 1, 1, 1) failed (returned NULL)\n");
 	else
 	{
@@ -1229,8 +1258,7 @@ int main(int ac, char **av)
 			++errors, printf("Test26: mem_destroy_space(1, 1, 1, 1) failed\n");
 	}
 
-	space4 = mem_create_space(sizeof(int), 10, 10, 10, 10, 0);
-	if (!space4)
+	if (!(space4 = mem_create_space(sizeof(int), 10, 10, 10, 10, 0)))
 		++errors, printf("Test27: mem_create_space(10, 10, 10, 10) failed (returned NULL)\n");
 	else
 	{
@@ -1259,8 +1287,7 @@ int main(int ac, char **av)
 
 	/* Test 5D space allocation and deallocation */
 
-	space5 = mem_create_space(sizeof(int), 1, 1, 1, 1, 1, 0);
-	if (!space5)
+	if (!(space5 = mem_create_space(sizeof(int), 1, 1, 1, 1, 1, 0)))
 		++errors, printf("Test30: mem_create_space(1, 1, 1, 1, 1) failed (returned NULL)\n");
 	else
 	{
@@ -1273,8 +1300,7 @@ int main(int ac, char **av)
 			++errors, printf("Test32: mem_destroy_space(1, 1, 1, 1, 1) failed\n");
 	}
 
-	space5 = mem_create_space(sizeof(int), 10, 10, 10, 10, 10, 0);
-	if (!space5)
+	if (!(space5 = mem_create_space(sizeof(int), 10, 10, 10, 10, 10, 0)))
 		++errors, printf("Test33: mem_create_space(10, 10, 10, 10, 10) failed (returned NULL)\n");
 	else
 	{
@@ -1305,8 +1331,7 @@ int main(int ac, char **av)
 
 	/* Test element sizes smaller than sizeof(void *) */
 
-	space2d = mem_create_space(sizeof(char), 1, 1, 0);
-	if (!space2d)
+	if (!(space2d = mem_create_space(sizeof(char), 1, 1, 0)))
 		++errors, printf("Test36: mem_create_space(char, 1, 1) failed (returned NULL)\n");
 	else
 	{
@@ -1319,8 +1344,7 @@ int main(int ac, char **av)
 			++errors, printf("Test38: mem_destroy_space(char, 1, 1) failed\n");
 	}
 
-	space2d = mem_create_space(sizeof(char), 10, 10, 0);
-	if (!space2d)
+	if (!(space2d = mem_create_space(sizeof(char), 10, 10, 0)))
 		++errors, printf("Test39: mem_create_space(char, 10, 10) failed (returned NULL)\n");
 	else
 	{
@@ -1345,8 +1369,7 @@ int main(int ac, char **av)
 
 	/* Test element sizes larger than sizeof(void *) */
 
-	space3d = mem_create_space(sizeof(double), 1, 1, 1, 0);
-	if (!space3d)
+	if (!(space3d = mem_create_space(sizeof(double), 1, 1, 1, 0)))
 		++errors, printf("Test42: mem_create_space(double, 1, 1, 1) failed (returned NULL)\n");
 	else
 	{
@@ -1359,8 +1382,7 @@ int main(int ac, char **av)
 			++errors, printf("Test44: mem_destroy_space(double, 1, 1, 1) failed\n");
 	}
 
-	space3d = mem_create_space(sizeof(double), 10, 10, 10, 0);
-	if (!space3d)
+	if (!(space3d = mem_create_space(sizeof(double), 10, 10, 10, 0)))
 		++errors, printf("Test45: mem_create_space(double, 10, 10, 10) failed (returned NULL)\n");
 	else
 	{
@@ -1387,8 +1409,7 @@ int main(int ac, char **av)
 
 	/* Test mem_space_start() */
 
-	space4 = mem_create_space(sizeof(int), 2, 3, 4, 5, 0);
-	if (!space4)
+	if (!(space4 = mem_create_space(sizeof(int), 2, 3, 4, 5, 0)))
 		++errors, printf("Test48: mem_create_space(int, 2, 3, 4, 5) failed (returned NULL)\n");
 	else
 	{
@@ -1412,7 +1433,7 @@ int main(int ac, char **av)
 			++errors;
 
 		start = mem_space_start(sizeof(int), 2, 3, 4, 5, 0);
-		memset((char *)space4 + start, nul, sizeof(int) * 2 * 3 * 4 * 5);
+		memset((char *)space4 + start, 0, sizeof(int) * 2 * 3 * 4 * 5);
 
 		for (i = 0; i < 2; ++i)
 			for (j = 0; j < 3; ++j)
@@ -1445,8 +1466,7 @@ int main(int ac, char **av)
 	/* Test pool functions */
 
 	start_clock = clock();
-	pool = pool_create(1024 * 1024);
-	if (!pool)
+	if (!(pool = pool_create(1024 * 1024)))
 		++errors, printf("Test53: pool_create(1024 * 1024) failed: %s\n", strerror(errno));
 	else
 	{
@@ -1461,7 +1481,7 @@ int main(int ac, char **av)
 
 		errno = 0;
 		if (pool_alloc(pool, 1) != NULL || errno != ENOSPC)
-			++errors, printf("Test55: pool_alloc(pool, 1) failed (errno %d, not %d)\n", errno, EINVAL);
+			++errors, printf("Test55: pool_alloc(pool, 1) failed (errno %d, not %d)\n", errno, ENOSPC);
 
 		errno = 0;
 		if (pool_alloc(NULL, 1) != NULL || errno != EINVAL)
@@ -1495,11 +1515,12 @@ int main(int ac, char **av)
 
 	if (!no_secure_mem)
 	{
-#ifdef MLOCK_NEEDS_PAGE_BOUNDARY
+#ifdef MLOCK_REQUIRES_PAGE_BOUNDARY
 		/* Test that page boundary is a power of two */
 
-		long pagesize = sysconf(_SC_PAGESIZE);
-		if (pagesize == -1)
+		long pagesize;
+		
+		if ((pagesize = sysconf(_SC_PAGESIZE)) == -1)
 			++errors, printf("Test59: Failed to perform test: sysconf(_SC_PAGESIZE) failed\n");
 		else
 		{
@@ -1515,8 +1536,7 @@ int main(int ac, char **av)
 		}
 #endif
 
-		mem1 = mem_create_secure(1024);
-		if (!mem1)
+		if (!(mem1 = mem_create_secure(1024)))
 			++errors, printf("Test60: mem_create_secure(1024) failed: %s\n", strerror(errno));
 		else
 		{
@@ -1525,15 +1545,13 @@ int main(int ac, char **av)
 				++errors, printf("Test61: mem_destroy_secure(1024) failed: mem == %p, not NULL\n", (void *)mem1);
 		}
 
-		pool = pool_create_secure(32);
-		if (!pool)
+		if (!(pool = pool_create_secure(32)))
 			++errors, printf("Test62: pool_create_secure(32) failed: %s\n", strerror(errno));
 		else
 		{
 			void *whitebox = pool->pool;
 
-			mem1 = pool_alloc(pool, 32);
-			if (!mem1)
+			if (!(mem1 = pool_alloc(pool, 32)))
 				++errors, printf("Test63: pool_alloc(pool, 32) failed: %s\n", strerror(errno));
 
 			pool_destroy_secure(&pool);
@@ -1545,8 +1563,15 @@ int main(int ac, char **av)
 		}
 	}
 
+	/* Test assumption: memory failure results in errno == ENOMEM */
+
+	errno = 0;
+	str = NULL;
+	if (!mem_resize(&str, UINT_MAX) && errno != ENOMEM)
+		++errors, printf("Test66: assumption failed: realloc failed but errno == \"%s\" (not \"%s\")\n", strerror(errno), strerror(ENOMEM));
+
 	if (errors)
-		printf("%d/65 tests failed\n", errors);
+		printf("%d/66 tests failed\n", errors);
 	else
 		printf("All tests passed\n");
 
@@ -1554,10 +1579,10 @@ int main(int ac, char **av)
 	{
 		printf("\n");
 		printf("    Note: Can't perform secure memory tests.\n");
-		printf("    Rerun test as root.\n");
+		printf("    Audit the code and rerun the test as root.\n");
 	}
 
-	return 0;
+	return (errors == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
 }
 
 #endif
